"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
var __await = (this && this.__await) || function (v) { return this instanceof __await ? (this.v = v, this) : new __await(v); }
var __asyncGenerator = (this && this.__asyncGenerator) || function (thisArg, _arguments, generator) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var g = generator.apply(thisArg, _arguments || []), i, q = [];
    return i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i;
    function verb(n) { if (g[n]) i[n] = function (v) { return new Promise(function (a, b) { q.push([n, v, a, b]) > 1 || resume(n, v); }); }; }
    function resume(n, v) { try { step(g[n](v)); } catch (e) { settle(q[0][3], e); } }
    function step(r) { r.value instanceof __await ? Promise.resolve(r.value.v).then(fulfill, reject) : settle(q[0][2], r); }
    function fulfill(value) { resume("next", value); }
    function reject(value) { resume("throw", value); }
    function settle(f, v) { if (f(v), q.shift(), q.length) resume(q[0][0], q[0][1]); }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DataTextSearcher = void 0;
const get_1 = __importDefault(require("lodash/get"));
const isEmpty_1 = __importDefault(require("lodash/isEmpty"));
const json_1 = require("../../../services/json");
const config_1 = require("./config");
class AbstractIterator {
    constructor(_queue, config) {
        this._queue = _queue;
        this._scanCount = config.scanCount || config_1.DEFAULT_TEXT_SEARCH_SCAN_COUNT;
    }
    _extractJobsData(ids) {
        return __awaiter(this, void 0, void 0, function* () {
            const pipeline = this._queue.client.pipeline();
            ids.forEach(id => pipeline.hmget(this._queue.toKey(id), 'data'));
            const data = yield pipeline.exec();
            return data.reduce((acc, [error, [jobData]], idx) => {
                if (!error && jobData) {
                    acc.push({
                        data: jobData,
                        id: ids[idx],
                    });
                }
                return acc;
            }, []);
        });
    }
}
class SetIterator extends AbstractIterator {
    constructor(queue, _key, config) {
        super(queue, config);
        this._key = _key;
    }
    generator() {
        return __asyncGenerator(this, arguments, function* generator_1() {
            var e_1, _a;
            this._stream = this._queue.client.zscanStream(this._key, {
                count: this._scanCount,
            });
            try {
                for (var _b = __asyncValues(this._stream), _c; _c = yield __await(_b.next()), !_c.done;) {
                    const ids = _c.value;
                    this._stream.pause();
                    const filteredIds = ids.filter((_k, idx) => !(idx % 2));
                    const data = yield __await(this._extractJobsData(filteredIds));
                    yield yield __await(data);
                    this._stream.resume();
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return)) yield __await(_a.call(_b));
                }
                finally { if (e_1) throw e_1.error; }
            }
        });
    }
    destroy() {
        if (this._stream) {
            this._stream.destroy();
        }
    }
}
class ListIterator extends AbstractIterator {
    constructor(queue, _key, config) {
        super(queue, config);
        this._key = _key;
        this._cursor = 0;
    }
    generator() {
        return __asyncGenerator(this, arguments, function* generator_2() {
            this._ids = yield __await(this._queue.client.lrange(this._key, 0, -1));
            while (true) {
                try {
                    const ids = this._nextChunk;
                    if (isEmpty_1.default(ids)) {
                        return yield __await(void 0);
                    }
                    const data = yield __await(this._extractJobsData(ids));
                    this._incCursor(data.length);
                    yield yield __await(data);
                }
                catch (e) {
                    console.error(e);
                    return yield __await(void 0);
                }
            }
        });
    }
    // noop
    destroy() { }
    _incCursor(n) {
        this._cursor += n;
    }
    get _nextChunk() {
        return this._ids.slice(this._cursor, this._cursor + this._scanCount);
    }
}
class DataTextSearcher {
    constructor(_queue) {
        this._queue = _queue;
    }
    search(args) {
        var e_2, _a;
        return __awaiter(this, void 0, void 0, function* () {
            const it = this._getIterator(args);
            const start = args.offset;
            const end = args.limit + start;
            const acc = [];
            try {
                // https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Statements/label
                mainLoop: for (var _b = __asyncValues(it.generator()), _c; _c = yield _b.next(), !_c.done;) {
                    const jobs = _c.value;
                    for (const job of jobs) {
                        const matched = this._matchData(job.data, args.term, args.key);
                        if (matched) {
                            acc.push(job.id);
                        }
                        if (acc.length >= end) {
                            break mainLoop;
                        }
                    }
                }
            }
            catch (e_2_1) { e_2 = { error: e_2_1 }; }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return)) yield _a.call(_b);
                }
                finally { if (e_2) throw e_2.error; }
            }
            it.destroy();
            const jobs = yield Promise.all(acc.slice(start, end).map(id => this._queue.getJob(id)));
            return jobs;
        });
    }
    _getIterator(args) {
        const redisKey = this._queue.toKey(args.status);
        const config = {
            scanCount: args.scanCount,
        };
        switch (args.status) {
            case 'completed':
            case 'failed':
            case 'delayed':
                return new SetIterator(this._queue, redisKey, config);
            case 'active':
            case 'waiting':
            case 'paused':
                return new ListIterator(this._queue, redisKey, config);
        }
    }
    _matchData(data, term, key) {
        if (key) {
            const parsedData = json_1.JsonService.maybeParse(data);
            if (typeof parsedData === 'object') {
                const value = String(get_1.default(parsedData, key));
                return value === null || value === void 0 ? void 0 : value.includes(term);
            }
        }
        else {
            return data.includes(term);
        }
    }
}
exports.DataTextSearcher = DataTextSearcher;
//# sourceMappingURL=data-text-search.js.map