import { DataSource } from 'apollo-datasource';
import type { Queue as BullQueue, JobCounts, JobStatus, JobId, Job } from 'bull';
import { CreateJobInput, JobDataSearchInput, MutationCleanQueueArgs, MutationCloseQueueArgs, MutationDiscardJobArgs, MutationEmptyQueueArgs, MutationLogArgs, MutationMoveJobToCompletedArgs, MutationMoveJobToFailedArgs, MutationPromoteJobArgs, MutationRemoveJobArgs, MutationRemoveJobsArgs, MutationRemoveJobsByPatternArgs, MutationResumeQueueArgs, MutationRetryJobArgs, MutationRetryJobsArgs, MutationUpdateJobDataArgs, OrderEnum } from '../../../typings/gql';
import { Maybe } from '../../../typings/utils';
import redisInfo from 'redis-info';
declare type Config = {
    textSearchScanCount?: number;
};
export declare enum ErrorEnum {
    QUEUE_NOT_FOUND = "Queue not found",
    JOB_NOT_FOUND = "Job not found",
    DATA_SEARCH_STATUS_REQUIRED = "Job status is required for data search",
    BAD_OFFSET = "Offset should be >= 0",
    BAD_LIMIT = "Limit should be >= 1"
}
export declare class BullDataSource extends DataSource {
    private _queues;
    private _config;
    private _queuesMap;
    constructor(_queues: BullQueue[], _config: Config);
    private _convertQueuesToMap;
    private _throwInternalError;
    private _throwQueueNotFound;
    private _throwJobNotFound;
    getQueueByName(name: string, throwIfNotFound?: boolean): BullQueue<any> | undefined;
    getQueues(): BullQueue[];
    getQueueJobs({ queue, limit, offset, status, id, ids, order, dataSearch, }: {
        limit?: number;
        offset?: number;
        dataSearch?: JobDataSearchInput;
        id?: string;
        order?: OrderEnum;
        ids?: string[];
        status?: JobStatus;
        queue: string;
    }): Promise<Job<any>[]>;
    private _filterJobs;
    getJob(queueName: string, id: JobId, throwIfNotFound?: boolean): Promise<Job<any> | null | undefined>;
    getJobLogs(queueName: string, id: number): Promise<{
        logs: string[];
        count: number;
    } | undefined>;
    getQueueJobsCounts(name: string): Promise<Maybe<JobCounts>>;
    getQueueFailedCount(name: string): Promise<Maybe<number>>;
    getQueueCompletedCount(name: string): Promise<Maybe<number>>;
    getQueueDelayedCount(name: string): Promise<Maybe<number>>;
    getQueueActiveCount(name: string): Promise<Maybe<number>>;
    getQueueWaitingCount(name: string): Promise<Maybe<number>>;
    getQueuePausedCount(name: string): Promise<Maybe<number>>;
    getQueueWaitingOrDelayedJobsCount(name: string): Promise<Maybe<number>>;
    getRedisInfo(): Promise<redisInfo.RedisInfo | null>;
    createJob({ queue: queueName, name, data, options, }: CreateJobInput): Promise<Job<any> | undefined>;
    removeJobsByPattern(args: MutationRemoveJobsByPatternArgs): Promise<boolean>;
    pauseQueue(name: string): Promise<BullQueue<any> | undefined>;
    cleanQueue(args: MutationCleanQueueArgs): Promise<Job<any>[] | undefined>;
    emptyQueue(args: MutationEmptyQueueArgs): Promise<BullQueue<any> | undefined>;
    closeQueue(args: MutationCloseQueueArgs): Promise<BullQueue<any> | undefined>;
    resumeQueue(args: MutationResumeQueueArgs): Promise<BullQueue<any> | undefined>;
    promoteJob(args: MutationPromoteJobArgs): Promise<Job<any> | null | undefined>;
    discardJob(args: MutationDiscardJobArgs): Promise<Job<any> | null | undefined>;
    updateJobData(args: MutationUpdateJobDataArgs): Promise<Job<any> | null | undefined>;
    createJobLog(args: MutationLogArgs): Promise<Job<any> | null | undefined>;
    retryJob(args: MutationRetryJobArgs): Promise<Job<any> | null | undefined>;
    retryJobs(args: MutationRetryJobsArgs): Promise<(Job<any> | null | undefined)[]>;
    removeJobById(args: MutationRemoveJobArgs): Promise<Job<any> | null | undefined>;
    removeJobs(args: MutationRemoveJobsArgs): Promise<(Job<any> | null | undefined)[]>;
    moveJobToCompleted(args: MutationMoveJobToCompletedArgs): Promise<Job<any> | null | undefined>;
    moveJobToFailed(args: MutationMoveJobToFailedArgs): Promise<Job<any> | null | undefined>;
}
export {};
